# -*- coding: utf-8 -*-
"""
Created on Wed Aug 19 22:51:03 2020

@author: jorritbleeker
"""

# Import required libraries for plotting and data handling
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sklearn.linear_model import LinearRegression

from labellines import labelLines

# Below are options for plotting
Plot_IVt = True  # Plot all Current/Voltage/Time graphs
Plot_4 = False  # Plot 4 temperatures instead of all 8 (True = 4, False = 8)
Plot_Line = True  # Plot line, or scatter points (True = line, False = scatter)
Plot_Tavg = False  # Type of temperature in legend (True = avg, False = Outflow)


# Section below is to import the datafiles
Current_Data = []
Voltage_Data = []
Temperature_Data = []
Power_Data = []
T_Legend = []
T_column = np.zeros((8,1))
ElectrodeArea = np.pi*(5.25)**2  # Radius of exposed electrode is 5,25 cm

# Thermocouple names taken from Reichert's thesis figure 5.6
Thermocouple_names = ['HotFeIn', 'HotFeOut', 'ColdFeIn', 'ColdFeOut', 'HotIIn',
                      'HotIOut', 'ColdIIn', 'ColdIOut', 'BeforeHB', 'AfterHB']
Temperatures = np.linspace(20, 90, 8, dtype=int)
for T in Temperatures:
    # Import potentiostat data
    IV_filename = '12currents'+str(T)+'C.txt'
    IV_Data = (np.loadtxt(IV_filename))
    Time_Data = IV_Data[:, 0]
    Current_Data.append(IV_Data[:, 1])
    Voltage_Data.append(IV_Data[:, 2])
    # Powerpoints is an array with the coordinates of the relaxed potential
    # in the graphs, and can be used in the power calculation
    Power_points = (175+65*np.linspace(1, 12, 12, dtype=int))*20-5
    Power_Data.append(IV_Data[Power_points, 1:3])
    # Import thermocouple data
    T_filename = str(T) + 'C.txt'

    Temperature_Data.append(pd.read_csv(T_filename, sep="\t", decimal=",",
                                        names=Thermocouple_names))

# Prepare to plot current on primary y axis, and voltage on secondary axis
# The for loop below makes a I & V vs t plot for all the temperatures
for i in range(0, 8):
    T_data = Temperature_Data[i]
    T_Hot_avg = T_data[["HotFeIn", "HotIIn",
                        "HotFeOut", "HotIOut"]].mean().mean()
    T_Hot_out = T_data[["HotFeOut", "HotIOut"]].mean().mean()
    T_Cold_avg = T_data[["ColdFeIn", "ColdIIn",
                         "ColdFeOut", "ColdIOut"]].mean().mean()
    T_Cold_out = T_data[["ColdFeOut", "ColdIOut"]].mean().mean()
    print(f"\nFor Heat-bath temperature of {Temperatures[i]} C")
    print(f"\tThe average Cell dT is: {T_Hot_avg-T_Cold_avg:.2f} C")
    print(f"\tThe average Outflow dT is: {T_Hot_out-T_Cold_out:.2f} C")
      
    if Plot_Tavg:
        T_Legend.append(f"{T_Hot_avg-T_Cold_avg:.0f}")
        T_column[i] = T_Hot_avg-T_Cold_avg
    else:
        T_Legend.append(f"{T_Hot_out-T_Cold_out:.0f}")
        T_column[i] = T_Hot_out-T_Cold_out

    if Plot_IVt and i == 7:

        fig, ax1 = plt.subplots()

        color = '#00438a'  # Yale Blue
        # Get data for plotting and convert units to mV & mA
        Current = Current_Data[i][:]*1000
        Voltage = Voltage_Data[i][:]*1000
        Time = Time_Data

        ax1.set_xlabel('Time (s)', fontsize=12)
        ax1.set_ylabel(r"Current density (A/m$^2$)", color=color, fontsize=12)
        ax1.plot(Time, Current*10/ElectrodeArea, color=color)
        ax1.tick_params(axis='y', labelcolor=color)
        # The line below makes sure 0 is included in the axis
        # ax1.set_ylim(min(0, min(Current)-5), max(0, max(Current)+5))

        ax2 = ax1.twinx()  # instantiate a second axes that shares the same x-axis

        color = '#c1292e'  # Fire Engine Red
        # we already handled the x-label with ax1
        ax2.set_ylabel('Voltage (mV)', color=color,fontsize=12)
        ax2.plot(Time, Voltage, color=color)
        ax2.tick_params(axis='y', labelcolor=color)
        # The line below makes sure 0 is included in the axis
        ax2.set_ylim(min(0, min(Voltage)-5), max(0, max(Voltage)+5))

        fig.tight_layout()  # otherwise the right y-label is slightly clipped
        IVt_plottitle = r'$\Delta$T = 34 K'
        # IVt_plottitle = "Heatbath @"+str(Temperatures[i]) + " C"
        plt.title(IVt_plottitle)
        plt.show()


# The section below is to make a plot of the power vs current density
Power = []
ElectrodeArea = np.pi*(5.25)**2  # Radius of exposed electrode is 5,25 cm
plt.figure(dpi=1200)

for i in range(0, 8):
    # Power density is calculated with P = V*I/Area, Current density is I/Area,
    # multiplied by 10000 to make the units W/m2 & A/m2
    I_Power = np.append([0.0], Power_Data[i][:, 0])
    V_Power = np.append([0.0], Power_Data[i][:, 1])
    Power.append([I_Power/ElectrodeArea*1e4,
                 I_Power*V_Power*-1e4/ElectrodeArea])

# Loops below are plotting the power curves, with options included as if/else statements

# Loop below is for 4 lines
if Plot_4:
    for i in range(1, 8, 2):
        if Plot_Line:
            plt.plot(Power[i][0], Power[i][1], label=T_Legend[i])
        else:
            plt.scatter(Power[i][0], Power[i][1], label=T_Legend[i])
        # plt.legend()
    labelLines(plt.gca().get_lines(), zorder=2.5, xvals=(5, 20), fontsize=10)


# Loop below is for 8 lines
else:
    # Next line sets colors of the plot, change plt.cm.$$$ to colormap of choice
    plt.gca().set_prop_cycle(plt.cycler('color', plt.cm.brg(np.linspace(0, 0.5, 8))))
    T_Legend[7] = r'$\Delta$T: '+str(T_Legend[7]) +' K'
    #T_Legend = ['0','4','9','13','17','22','29','34']
    for i in range(0, 8):
        if Plot_Line:
            plt.plot(Power[i][0], Power[i][1], label=T_Legend[i], marker='o',
                     markersize =5)
        else:
            plt.scatter(Power[i][0], Power[i][1], label=T_Legend[i])
    # plt.legend(T_Legend)
    labelLines(plt.gca().get_lines(), zorder=2, xvals=(
        [10.5, 11.5, 12.3, 13, 14, 15.5, 16.5, 17.5]), fontsize=11)


#plt.text(18,0.65,r'$\Delta$T',fontsize = 14)
plt.xlabel(r"Current density (A/m$^2$)", fontsize=12)
plt.xlim(0,)
plt.ylim(-0.6, 0.8)
plt.ylabel(r"Power density (W/m$^2$)", fontsize=12)
plt.tick_params(axis='both', direction='in', top='True', right='True')
plt.show()


#create a line to fit the data at 1 current density
if False:
    pwr = np.zeros(8)
    for n in range(0,13): # n-th current density
        for i in range(0,8):
            pwr[i] = Power[i][1][n]
        T_column.reshape(-1,1)
        pwr.reshape(-1,1)
        
        lm = LinearRegression()
        lm.fit(T_column,pwr)
        P_fit = lm.predict(T_column)
        print(lm.score(T_column,pwr))
        
        plt.plot(T_column,pwr,'.')
        plt.plot(T_column,P_fit)
        plt.xlabel("dT (deg K)")
        plt.ylabel(r"Power density (W/m$^2$)")
        plt.show()
