# -*- coding: utf-8 -*-
"""
Created on Sun Sep 20 14:01:51 2020

This script is created to convert the CV graphs into a E1/2 vs T graph from
which the Seebeck coefficient can be extracted (RFB article figure S-8)

@author: jorritbleeker
"""
# Import required libraries
import numpy as np
import matplotlib.pyplot as plt

#function below is made to normalize current data between 0 and 1
def norm(A):
    dA = max(A)-min(A)
    A = (A-min(A))/dA
    return A
    
# Make arrays with the temperature points of Fe and I electrolytes
T_polyI = [22.1, 25.1, 29.6, 34.9, 41.0, 46.0, 50.6]
T_FeCN6 = [22.6, 26.0, 32.6, 37.5, 42.3, 46.6, 50.6, 54.8]

# Make arrays to store data in
Data_polyI = []
Data_FeCN6 = []
Seebeck_polyI = []
Seebeck_FeCN6 = []

"""
------------------------------------------------------------------------------
Section below deals with obtaining the PolyIodide CV data from the datafiles
------------------------------------------------------------------------------
"""
# This loop will load the data from the polyiodide CV scans
for T in T_polyI:
    filename = "Polyiodide 50 mvs " + str(T) + "C.ids"
    CV_Data = []
    # i is looped over the starting points of each CV scan
    for i in [303,1186,2069,2952]:
        CV_Scan = np.loadtxt(filename, encoding= 'unicode_escape', skiprows = i, max_rows=800)
        CV_Data.append(CV_Scan)
        maxIndex = np.argmax(CV_Scan[200:380,1])+200 #Max current between 0.4 and 0.58 V
        minIndex = np.argmin(CV_Scan[500:700,1])+500 #Min current between 0.5 and 0.3 V
        Ehalf = (CV_Scan[maxIndex,2]+CV_Scan[minIndex,2])/2 #redox potential vs Ag/AgCl @ 20 C
        Seebeck_polyI.append([T,Ehalf]) # store half reaction potential and temperature
    Data_polyI.append(CV_Data)

if True: #If true, plot CV scans, if False, do not
    plt.gca().set_prop_cycle(plt.cycler('color', plt.cm.brg(np.linspace(0, 1, 7))))
    CV_nr = 3 # decides which CV iteration is plotted (number between 0 and 3)
    for i in range(len(T_polyI)):
        CV_label = str(T_polyI[i]) + " C"
        plt.plot(Data_polyI[i][CV_nr][:,2],norm(Data_polyI[i][CV_nr][:,1]), label = CV_label)
    plt.xlabel("Potential (V vs Ag/AgCl @ 20C)")
    plt.ylabel("Current (Normalized)")
    plt.title("Polyiodide")
    plt.legend()
    plt.show()

"""
------------------------------------------------------------------------------
Section below deals with obtaining the Ferrocyanide CV data from the datafiles
------------------------------------------------------------------------------
"""
    
# This loop will load the data from the ferrocyanide CV scans
for T in T_FeCN6:
    filename = "FeCN6 50 mvs " + str(T) + "C.ids"
    CV_Data = []
    # i is looped over the starting points of each CV scan
    for i in [303,1786,3269,4752]:
        CV_Scan = np.loadtxt(filename, encoding= 'unicode_escape', skiprows = i, max_rows=1400)
        CV_Data.append(CV_Scan)
        maxIndex = np.argmax(CV_Scan[100:400,1])+100 #Max current between 0.1 and 0.4 V
        minIndex = np.argmin(CV_Scan[700:1000,1])+700 #Min current between 0.3 and 0.0 V
        Ehalf = (CV_Scan[maxIndex,2]+CV_Scan[minIndex,2])/2 #redox potential vs Ag/AgCl @ 20 C
        Seebeck_FeCN6.append([T,Ehalf]) # store half reaction potential and temperature
    Data_FeCN6.append(CV_Data)

if True: #If true, plot CV scans, if False, do not
    plt.gca().set_prop_cycle(plt.cycler('color', plt.cm.brg(np.linspace(0, 1, 8))))
    CV_nr = 1 # decides which CV iteration is plotted (number between 0 and 3)
    for i in range(len(T_FeCN6)):
        CV_label = str(T_FeCN6[i]) + " C"
        plt.plot(Data_FeCN6[i][CV_nr][:,2],norm(Data_FeCN6[i][CV_nr][:,1]), label = CV_label)
    plt.xlabel("Potential (V vs Ag/AgCl @ 20C)")
    plt.ylabel("Current (Normalized)")
    plt.title("Fe(CN)6")
    plt.legend()
    plt.show()

"""
------------------------------------------------------------------------------
The section below deals with calculation of the Seebeck coefficient
------------------------------------------------------------------------------
"""
# Convert seebeck data into an numpy array for calculations and plotting
Seebeck_polyI = np.array(Seebeck_polyI, dtype = 'float64')
Seebeck_FeCN6 = np.array(Seebeck_FeCN6, dtype = 'float64')

# Temperature range over which the fits will be plotted (20 - 60 C)
fit_T = np.linspace(20,60,100)

# Linear regression of polyiodine Seebeck dataset
coeff_polyI = np.polyfit(Seebeck_polyI[:,0],Seebeck_polyI[:,1],1)
func_polyI = np.poly1d(coeff_polyI)
fit_polyI = func_polyI(fit_T)

# Linear regression of ferrocyanide Seebeck dataset
coeff_FeCN6 = np.polyfit(Seebeck_FeCN6[:,0],Seebeck_FeCN6[:,1],1)
func_FeCN6 = np.poly1d(coeff_FeCN6)
fit_FeCN6 = func_FeCN6(fit_T)


# Plot temperature (x) vs half reaction potential (y) for polyiodide
plt.scatter(Seebeck_polyI[:,0],Seebeck_polyI[:,1], facecolors='none', edgecolors='y')
plt.plot(fit_T,fit_polyI, 'black')
# Add triangle to indicate slope
i1 = 35
i2 = 55
X1 = [fit_T[i1], fit_T[i2]]
Y1 = [fit_polyI[i1],fit_polyI[i1]]
X2 = [fit_T[i2], fit_T[i2]]
Y2 = [fit_polyI[i1],fit_polyI[i2]]
plt.plot(X1,Y1, color = 'black')
plt.plot(X2,Y2, color = 'black')
plt.text(42.5,0.448,f'+{coeff_polyI[0]*1000:.2f} mV/K')
plt.xlabel(r'Temperature ($\degree$C)')
plt.ylabel(r'E$_{1/2}$ (V vs Ag/AgCl)')
plt.show()

# Plot temperature (x) vs half reaction potential (y) for ferrocyanide
plt.scatter(Seebeck_FeCN6[:,0],Seebeck_FeCN6[:,1], facecolors='none', edgecolors='g')
plt.plot(fit_T,fit_FeCN6, 'black')

# Add triangle to indicate slope
i1 = 35
i2 = 55
X1 = [fit_T[i1], fit_T[i1]]
Y1 = [fit_FeCN6[i1],fit_FeCN6[i2]]
X2 = [fit_T[i1], fit_T[i2]]
Y2 = [fit_FeCN6[i2],fit_FeCN6[i2]]
plt.plot(X1,Y1, color = 'black')
plt.plot(X2,Y2, color = 'black')
plt.text(26,0.19,f'{coeff_FeCN6[0]*1000:.2f} mV/K')
plt.xlabel(r'Temperature ($\degree$C)')
plt.ylabel(r'E$_{1/2}$ (V vs Ag/AgCl @20C)')
plt.show()


    