# -*- coding: utf-8 -*-
"""
Created on Wed Sep 16 22:42:13 2020

@author: jorritbleeker
This script is used to make the figure S-4 of the RFB article
I had trouble finding the original measurement data that Stijn used.

The concentrations used in this measurement are:
Ferrocyanide electrolyte:
    K4Fe(CN)6:  0.3 M
    K3Fe(CN)6:  0.3 M
    KCl:        0.35 M
Iodine electrolyte:
    I2:         0.1 M
    KI:         0.3 M
    KCl:        0.7 M
    
The datapoints were taken from "Resistance Data.xls" tab "24-11",
 which Stijn used to make figure 4.3 in his thesis.

The resitance vs temperature function is fitted according to:
    R(T) = Rref/(1+theta(T-Tref)) 
    Here theta is a temperature coefficient
Reference: M.Mäntynen, “Temperature correction coefficients of electrical 
conductivity and of density measurements for saline groundwater.”
"""
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit

def Rfunc(T, theta):
    #Returns R(T), R0 & T0 are the middle datapoints of the dataset
    return 7.0829/(1.+theta*(T-21.8714))

# Temperature datapoints (in degree Celcius)
T = np.array([21.8714, 25.0739, 28.0951, 31.04235, 30.2453, 33.54205,
              36.9453, 40.1109])

# Resistance values at the temperatures above (in ohms)
R = np.array([7.0829, 5.5148, 5.0584, 4.4166, 4.9059, 4.0976, 3.8474, 3.593])


# Use scipy curve_fit to fit the data to the formula and find temperature coef
# 0.06 is the initial guess as per thesis Steijn
theta_fit, covariance = curve_fit(Rfunc,T,R,0.06)

plt.scatter(T,R, facecolors='none', edgecolors='r')
plt.ylabel(r'Ohmic resistance ($\Omega$ cm$^2$)', fontsize = 12)
plt.xlabel(r'Temperature ($\degree$C)', fontsize = 12)
plt.tick_params(axis='both',direction='in', top = 'True',right = 'True')

# Create a row of temperature values to fit the Resistance to
T_fit = np.linspace(20,45,50)
R_fit = Rfunc(T_fit,theta_fit)
plt.plot(T_fit,R_fit, color='black')

plt.text(24,6.5, fr"$\theta$ = {theta_fit[0]:.3f} K⁻¹")
plt.text(19.5,3.2, r'$R(T) = \dfrac{R_0}{1 + \theta(T - T_0)}$', fontsize = 12)

plt.show()



