# -*- coding: utf-8 -*-
"""
Created on Mon Aug 31 16:10:04 2020

This script is created to make a V dQ graph for a PolyIodide/Ferrocyanide RFB
Assumed all reactions follow Nernst's Law
Discharging: I2 + 2Fe(CN)6 4-  -> 2I- + 2Fe(CN)6 3-
Charging: 2I- + 2Fe(CN)6 3- -> I2 + 2Fe(CN)6 4-

This script is the same as used for 4a/4b, but modified to loop over various
Seebeck coefficients

@author: jorritbleeker
"""

# Import required libraries
import numpy as np
import matplotlib.pyplot as plt
from labellines import labelLines

# Define variables and properties of system (feel free to tweek)
C0_FeCN6_3 = 0.3    #FeCN6 3- starting concentration in mol/L
C0_FeCN6_4 = 0.3    #FeCN6 4- starting concentration in mol/L
C0_I3 = 0.3         #Iodine starting concentration in mol/L
C0_KI = 0.9         #Iodide starting concentration in mol/L
T_hot = 80          #Temperature of hot cell in degrees Celcius
T_cold = 20         #Temperature of cold cell in degrees Celcius
alpha_cell = +2.88  #Seebeck coefficient of cell in mV/K
dQ_min = 5e-3       #Lowest concentration allowed in cell in mol/L
res = 1000          #Graph resolution, in amount of points           

# Define constants and chemical properties associated with systems
E0_FeCN6 = 0.36     #Standard electrode potential vs NHE in V 
E0_I = 0.54         #Standard electrode potential vs NHE in V 
R = 8.314           #Gas constant in J/molK
F = 96485           #Faraday constant in C/mol e-
T0 = 298            #Standard temperature in degrees Kelvin
n = 1               #Reaction written for 1 electron (0.5 I3, 1.5 I-, 1 all else)


# Calculate a few variables
E_Cell0 = E0_I - E0_FeCN6           #Standard cell potential
T_hot = T_hot + 273.15              #Convert units to K
T_cold = T_cold + 273.15            #Convert units to K
C0_I = C0_KI - C0_I3                #Part of iodide is used in tri-iodide

#Prepare list and legend to loop the Seebeck coefficient over
Seebecks = [1.0,2.88,5.0,10.0]
alpha_legend = ['1.0','2.88','5',r'$\alpha$ = 10 mV/K']
k=0 # is used as index of the legend
plt.figure(dpi=1200)
plt.gca().set_prop_cycle(plt.cycler('color', plt.cm.brg(np.linspace(0, 0.5, 4))))

# Determine which compound is limiting for discharging
if C0_I3 <= 0.5*C0_FeCN6_4:
    dis_Q = 2*C0_I3 #Factor 2, because electron I3 ratio is 2:1
else:
    dis_Q = C0_FeCN6_4

# Determine which compound is limiting for charging
if C0_I <= 1.5*C0_FeCN6_3:
    ch_Q = (2/3)*C0_I # Factor 2/3, because electron to I- ratio is 2:3
else:
    ch_Q = C0_FeCN6_3

# Make a concentration range for all species, dQ is a logaritmically spaced 
# charge distribution from 0 to ch_Q+dis_Q, within min_dQ at both sides
dQ = np.linspace(0+dQ_min,ch_Q+dis_Q-dQ_min, res)

# When dQ is maximum, the system is charged and FeCN6 4- & I2 are most abundant
C_FeCN6_4 = dQ - dis_Q + C0_FeCN6_4
C_I3 = 0.5*dQ - dis_Q*0.5 + C0_I3 # 2e- : 1I2 ratio

# When dQ is maximum, the system is charged and FeCN6 4- & I2 are least abundant
C_FeCN6_3 = -1*dQ + dis_Q + C0_FeCN6_3
C_I = -1.5*dQ + dis_Q*1.5 + C0_I

# Normalize charge extend
dQ_norm = dQ/(ch_Q + dis_Q)
for alpha_cell in Seebecks:
    alpha_cell = alpha_cell/1000    #Convert units to V/K
    # Calculate reduction potentials, and cell potential for T_cold
    E_I_cold = E0_I + R*T_cold/n/F*np.log(C_I3**0.5/C_I**1.5)
    E_Fe_cold = E0_FeCN6 + R*T_cold/n/F*np.log(C_FeCN6_3/C_FeCN6_4)
    
    Ecell_cold = E_I_cold - E_Fe_cold + alpha_cell*(T_cold-T0) 
    

    # Calculate reduction potentials, and cell potential for T_hot
    E_I_hot = E0_I + R*T_hot/n/F*np.log(C_I3**0.5/C_I**1.5)
    E_Fe_hot = E0_FeCN6 + R*T_hot/n/F*np.log(C_FeCN6_3/C_FeCN6_4)
    
    Ecell_hot = E_I_hot - E_Fe_hot + alpha_cell*(T_hot-T0) 
    
    
    #Below text is to find the optimum operating points for maximum work
    W = np.zeros((res,res))
    for i in range(res):
        for j in range(i,res):
            W[i,j] = (Ecell_hot[i]-Ecell_cold[j])*(dQ[j]-dQ[i])*n*F
    
    
    
    opt_dQ = np.array([np.where(W == W.max())[0],np.where(W == W.max())[1]])
    
    
    """
    The section below here is to prepare the final figures of the RFB article
    - Continuous mode -
    
    Important note: dQ = state of charge, Q = energy
    
    The efficiency is defined as Qout/Qin, in this case:
    Qin is a sum of the below two factors:
        - Endothermic reaction: dS*T_h*dn       (dn is dQ in this script)
        - Extra heating due imperfect HeatEx: rho*Cp*V*dT*(1-efficiency)
    Qout is:
        - (E_hot-E-cold)*dQ*n*F (or W, as calculated in previous section)
    Note: Here E_hot/E_cold is the cell potential at the dQ at cell outlet
        
    """
    T_c = 20 + 273.15                       # Temperature of cold cell
    T_h = 80 + 273.15                       # Temperature of hot cell
    dS = alpha_cell*1*96485                 # alpha*n*F (units J/mol/K)
    rho = 1000                              # density (assumed water like) - kg/m3
    Cp = 4184                               # Heat capacity (assumed water like) - J/kg K
    V = 0.002                               # Volume - 2 L (1L of each electrolyte)
    HX_eff = 0.90                           # Heat exchanger efficiency
    
    dT = T_h - T_c
    eff_Carnot = dT/T_h*100                 # Carnot efficiency (T_hot-T_cold)/T_hot
    
    # The section below plots the efficiency versus delta state of charge
    # The delta state of charge is the difference between SOC in the hot and cold cell outlets
    # All values are calculated from 0.5, i.e. dSOC of 0.2 is a cell that is
    # charged to 0.6 and discharged to 0.4.
    # These values are taken from the W-matrix diagonal, and only the upper half
    # The lower half will always result in negative work
    # All arrays are res//2 in size, because only the upper diagonal is taken.   
    
    Q_out = np.zeros([res//2])  # Array to store produced energy in
    dQ_fig = np.zeros([res//2]) # Array to store state of charge values in
    Q_in = np.zeros([res//2])   # Array to store required energy values in
       
    # coordinates i,j are the diagonal in the top right quadrant, 
    # To start from the center, it starts at res//2, the -1 for j is because python starts counting at 0, not 1
    for i in range(res//2):
        j = res - i - res//2 - 1 
        Q_out[i] = W[j,i+res//2]
        dQ_fig[i] = dQ_norm[i+res//2]-dQ_norm[j]
        Q_in[i] = T_h*dS*(dQ[i+res//2]-dQ[j])
        Q_in[i] += (1-HX_eff)*dT*Cp*rho*V
    
    H2P_Eff = Q_out/Q_in
    
    plt.plot(dQ_fig,H2P_Eff*100, label = alpha_legend[k]) # multiply by 100 to make percentages
    k += 1

labelLines(plt.gca().get_lines(),zorder=2,xvals=(0.01,0.6), fontsize = 10, align = False)

if 1:    # if 1, plots carnot efficiency in the graph, if 0, not
    plt.plot([0,0.44],[eff_Carnot,eff_Carnot],':',color='black')
    plt.plot([0.57,1],[eff_Carnot,eff_Carnot],':',color='black')
    plt.text(0.45, eff_Carnot*0.99, 'Carnot')

plt.ylim(0,18)
plt.xlim(0,1)
plt.xlabel(r"$\Delta$State of Charge (Normalized)", fontsize = 10)
plt.ylabel("Heat to power efficiency (%)", fontsize = 10)
plt.tick_params(axis='both',direction='in', top = 'True',right = 'True')

plt.show()

    